"""
    Registry API

    Registry API enabling advanced search on PDS data and metadata. The API provides end-points to search for bundles, collections and any PDS products with advanced search queries. It also enables to browse the archive hierarchically downward (e.g. collection/s products) or upward (e.g. bundles containing a product). The detailed syntax for querying the end-point is given in the reference documentation.   # noqa: E501

    The version of the OpenAPI document: 1.1
    Contact: pds-operator@jpl.nasa.gov
    Generated by: https://openapi-generator.tech
"""


import re  # noqa: F401
import sys  # noqa: F401

from pds.api_client.api_client import ApiClient, Endpoint as _Endpoint
from pds.api_client.model_utils import (  # noqa: F401
    check_allowed_values,
    check_validations,
    date,
    datetime,
    file_type,
    none_type,
    validate_and_convert_types
)
from pds.api_client.model.error_message import ErrorMessage


class ClassesApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

        def __class_list(
            self,
            _class,
            **kwargs
        ):
            """returns all PDS products of the given PDS product class that meet all given constraints.   # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.class_list(_class, async_req=True)
            >>> result = thread.get()

            Args:
                _class (str): syntax: one of the allowable enum values  This is a shortcut for doing a query with ... 

            Keyword Args:
                fields ([str]): syntax: fields=field1,field2,...  behavior: this parameter and the headder Accept: type determine what content is packaged for the result. While the types application/csv, application/kvp+json, and text/csv return only the fields requesteted, all of the other types have a minimal set of fields that must be returned. Duplicating a minimally required field in this parameter has not effect. The types vnd.nasa.pds.pds4+json and vnd.nasa.pds.pds4+xml have a complete set of fields that must be returned; meaning this parameter does not impact their content. When fields is not used, then the minimal set of fields, or all when minimal is an empty set, is returned.  notes: the blob fields are blocked unless specifically requrested and only for the */csv and application/kvp+csv types. . [optional]
                keywords ([str]): syntax: keyword=keyword1,keyword2,...  behavior: free text search on title and description (if set q is ignored . [optional]
                limit (int): syntax: limit=10  behavior: maximum number of matching results returned, for pagination  note: limit=0 returns just the summary . [optional] if omitted the server will use the default value of 100
                q (str): syntax: q=\"vid eq 13.0\"  behavior: query uses eq,ne,gt,ge,lt,le,(,),not,and,or operators. Properties are named as in 'properties' attributes, literals are strings between quotes, like \"animal\", or numbers. Detailed query specification is available at https://bit.ly/3h3D54T  note: ignored when keyword is present . [optional]
                sort ([str]): syntax: sort=asc(field0),desc(field1),...  behavior: is this implemented? . [optional]
                start (int): syntax: start=12  behavior: offset in matching result list, for pagination . [optional] if omitted the server will use the default value of 0
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (int/float/tuple): timeout setting for this request. If
                    one number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                {str: (bool, date, datetime, dict, float, int, list, str, none_type)}
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            kwargs['_class'] = \
                _class
            return self.call_with_http_info(**kwargs)

        self.class_list = _Endpoint(
            settings={
                'response_type': ({str: (bool, date, datetime, dict, float, int, list, str, none_type)},),
                'auth': [],
                'endpoint_path': '/classes/{class}',
                'operation_id': 'class_list',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                    '_class',
                    'fields',
                    'keywords',
                    'limit',
                    'q',
                    'sort',
                    'start',
                ],
                'required': [
                    '_class',
                ],
                'nullable': [
                ],
                'enum': [
                    '_class',
                ],
                'validation': [
                    'limit',
                    'start',
                ]
            },
            root_map={
                'validations': {
                    ('limit',): {

                        'inclusive_minimum': 0,
                    },
                    ('start',): {

                        'inclusive_minimum': 0,
                    },
                },
                'allowed_values': {
                    ('_class',): {

                        "ANY": "any",
                        "BUNDLES": "bundles",
                        "COLLECTIONS": "collections",
                        "DOCUMENTS": "documents",
                        "OBSERVATIONALS": "observationals",
                        "PRODUCTS": "products"
                    },
                },
                'openapi_types': {
                    '_class':
                        (str,),
                    'fields':
                        ([str],),
                    'keywords':
                        ([str],),
                    'limit':
                        (int,),
                    'q':
                        (str,),
                    'sort':
                        ([str],),
                    'start':
                        (int,),
                },
                'attribute_map': {
                    '_class': 'class',
                    'fields': 'fields',
                    'keywords': 'keywords',
                    'limit': 'limit',
                    'q': 'q',
                    'sort': 'sort',
                    'start': 'start',
                },
                'location_map': {
                    '_class': 'path',
                    'fields': 'query',
                    'keywords': 'query',
                    'limit': 'query',
                    'q': 'query',
                    'sort': 'query',
                    'start': 'query',
                },
                'collection_format_map': {
                    'fields': 'multi',
                    'keywords': 'multi',
                    'sort': 'multi',
                }
            },
            headers_map={
                'accept': [
                    '*',
                    '*/*',
                    'application/csv',
                    'application/json',
                    'application/kvp+json',
                    'application/vnd.nasa.pds.pds4+json',
                    'application/vnd.nasa.pds.pds4+xml',
                    'application/xml',
                    'text/csv',
                    'text/html',
                    'text/xml'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__class_list
        )

        def __classes(
            self,
            **kwargs
        ):
            """returns a list of the available PDS product class at this API. This is not all of the PDS product class available in the database, but simply the names that can be used in the /classes/{class} endpoints.   # noqa: E501

            This method makes a synchronous HTTP request by default. To make an
            asynchronous HTTP request, please pass async_req=True

            >>> thread = api.classes(async_req=True)
            >>> result = thread.get()


            Keyword Args:
                _return_http_data_only (bool): response data without head status
                    code and headers. Default is True.
                _preload_content (bool): if False, the urllib3.HTTPResponse object
                    will be returned without reading/decoding response data.
                    Default is True.
                _request_timeout (int/float/tuple): timeout setting for this request. If
                    one number provided, it will be total request timeout. It can also
                    be a pair (tuple) of (connection, read) timeouts.
                    Default is None.
                _check_input_type (bool): specifies if type checking
                    should be done one the data sent to the server.
                    Default is True.
                _check_return_type (bool): specifies if type checking
                    should be done one the data received from the server.
                    Default is True.
                _host_index (int/None): specifies the index of the server
                    that we want to use.
                    Default is read from the configuration.
                async_req (bool): execute request asynchronously

            Returns:
                [str]
                    If the method is called asynchronously, returns the request
                    thread.
            """
            kwargs['async_req'] = kwargs.get(
                'async_req', False
            )
            kwargs['_return_http_data_only'] = kwargs.get(
                '_return_http_data_only', True
            )
            kwargs['_preload_content'] = kwargs.get(
                '_preload_content', True
            )
            kwargs['_request_timeout'] = kwargs.get(
                '_request_timeout', None
            )
            kwargs['_check_input_type'] = kwargs.get(
                '_check_input_type', True
            )
            kwargs['_check_return_type'] = kwargs.get(
                '_check_return_type', True
            )
            kwargs['_host_index'] = kwargs.get('_host_index')
            return self.call_with_http_info(**kwargs)

        self.classes = _Endpoint(
            settings={
                'response_type': ([str],),
                'auth': [],
                'endpoint_path': '/classes',
                'operation_id': 'classes',
                'http_method': 'GET',
                'servers': None,
            },
            params_map={
                'all': [
                ],
                'required': [],
                'nullable': [
                ],
                'enum': [
                ],
                'validation': [
                ]
            },
            root_map={
                'validations': {
                },
                'allowed_values': {
                },
                'openapi_types': {
                },
                'attribute_map': {
                },
                'location_map': {
                },
                'collection_format_map': {
                }
            },
            headers_map={
                'accept': [
                    '*',
                    '*/*',
                    'application/csv',
                    'application/json',
                    'application/kvp+json',
                    'application/vnd.nasa.pds.pds4+json',
                    'application/vnd.nasa.pds.pds4+xml',
                    'application/xml',
                    'text/csv',
                    'text/html',
                    'text/xml'
                ],
                'content_type': [],
            },
            api_client=api_client,
            callable=__classes
        )
